     --==================================================================--

with Fc70b00;    -- Generic list abstraction.
with Cc70b01_0;  -- Generic "zeroing" operation for lists of discrete types.

with Report;
procedure Cc70b01 is

   type Points is range 0 .. 10;                    -- Discrete type.
   package Lists_Of_Scores is new Fc70b00 (Points); -- List-of-points
   -- abstraction.
   Scores : Lists_Of_Scores.List_Type;              -- List of points.

   procedure Reset_All_Scores is new                -- Operation on lists of
   Cc70b01_0 (Points, Lists_Of_Scores);           -- points.

   -- Begin test code declarations: -----------------------

   type Tc_Score_Array is array (1 .. 3) of Points;

   Tc_Initial_Values : constant Tc_Score_Array := (2, 4, 6);
   Tc_Final_Values   : constant Tc_Score_Array := (0, 0, 0);

   Tc_Correct_Initial_Values : Boolean := False;
   Tc_Correct_Final_Values   : Boolean := False;

   procedure Tc_Initialize_List (L : in out Lists_Of_Scores.List_Type) is
   begin                                  -- Initial list contains 3 scores
      for I in Tc_Score_Array'Range loop  -- with the values 2, 4, and 6.
         Lists_Of_Scores.Add_Element (L, Tc_Initial_Values (I));
      end loop;
   end Tc_Initialize_List;

   procedure Tc_Verify_List (L : in out Lists_Of_Scores.List_Type;
      Expected                 : in     Tc_Score_Array; Ok : out Boolean)
   is
      Actual : Tc_Score_Array;
   begin                                  -- Verify that all scores have been
      Lists_Of_Scores.Reset (L);          -- set to zero.
      for I in Tc_Score_Array'Range loop
         Lists_Of_Scores.Read_Element (L, Actual (I));
      end loop;
      Ok := (Actual = Expected);
   end Tc_Verify_List;

   -- End test code declarations. -------------------------

begin
   Report.Test
     ("CC70B01",
      "Check that a library-level generic subprogram " &
      "may have a formal package as a formal parameter, and that " &
      "the generic formal actual part may specify explicit actual " &
      "parameters. Check that a use clause is legal in the " &
      "generic formal part");

   Tc_Initialize_List (Scores);
   Tc_Verify_List (Scores, Tc_Initial_Values, Tc_Correct_Initial_Values);

   if not Tc_Correct_Initial_Values then
      Report.Failed ("List contains incorrect initial values");
   end if;

   Reset_All_Scores (Scores);
   Tc_Verify_List (Scores, Tc_Final_Values, Tc_Correct_Final_Values);

   if not Tc_Correct_Final_Values then
      Report.Failed ("List contains incorrect final values");
   end if;

   Report.Result;
end Cc70b01;
