-- CXE4001.A
--
--                             Grant of Unlimited Rights
--
--     Under contracts F33600-87-D-0337, F33600-84-D-0280, MDA903-79-C-0687,
--     F08630-91-C-0015, and DCA100-97-D-0025, the U.S. Government obtained
--     unlimited rights in the software and documentation contained herein.
--     Unlimited rights are defined in DFAR 252.227-7013(a)(19).  By making
--     this public release, the Government intends to confer upon all
--     recipients unlimited rights  equal to those held by the Government.
--     These rights include rights to use, duplicate, release or disclose the
--     released technical data and computer software in whole or in part, in
--     any manner and for any purpose whatsoever, and to have or permit others
--     to do so.
--
--                                    DISCLAIMER
--
--     ALL MATERIALS OR INFORMATION HEREIN RELEASED, MADE AVAILABLE OR
--     DISCLOSED ARE AS IS.  THE GOVERNMENT MAKES NO EXPRESS OR IMPLIED
--     WARRANTY AS TO ANY MATTER WHATSOEVER, INCLUDING THE CONDITIONS OF THE
--     SOFTWARE, DOCUMENTATION OR OTHER INFORMATION RELEASED, MADE AVAILABLE
--     OR DISCLOSED, OR THE OWNERSHIP, MERCHANTABILITY, OR FITNESS FOR A
--     PARTICULAR PURPOSE OF SAID MATERIAL.
--*
--
-- OBJECTIVE:
--      Check that exception propagation between partitions is properly
--      handled.  In particular check that:
--      a predefined exception can be raised in one partition and handled in
--      another;
--      an exception declared in a remote call interface library unit
--      can be raised in one partition and handled in another partition;
--      an exception declared in partition A and not visible to partition B
--      can be raised in partition A and handled in partition B with an others
--      clause;
--      an exception declared in a partition A and not visible to partition B
--      can be raised in partition A, propagated through partition B, and
--      handled back in A.
--
-- TEST DESCRIPTION:
--      This test is composed of the following compilation units:
--          CXE4001_Decl_Pure - pure package containing declarations
--                      shared between partitions A & B
--          CXE4001_Partition_A - rci package interface for partition A
--          CXE4001_Partition_B - rci package interface for partition B
--          CXE4001_A - main procedure for partition A
--          CXE4001_B - main procedure for partition B
--
-- SPECIAL REQUIREMENTS:
--      Compile the compilation units in this file.
--      Create the two partitions (A and B) with the following contents:
--        Partition A contains:
--           CXE4001_A  (main procedure)
--           CXE4001_Partition_A  (RCI package)
--           and all normal and pure packages with'ed by these units.
--        Partition B contains:
--           CXE4001_B  (main procedure)
--           CXE4001_Partition_B  (RCI package)
--           and all normal and pure packages with'ed by these units.
--        Note that package Report is included in both partitions.
--      Run the program by starting both partitions.  The partitions may be
--      started in either order.
--
-- APPLICABILITY CRITERIA:
--      This test applies only to implementations that:
--         support the Distribution Annex,
--         support the Remote_Call_Interface pragma, and
--         provide an implementation of System.RPC.
--
-- PASS/FAIL CRITERIA:
--      This test passes if and only if both partition A and
--      partition B print a message reporting that the test passed.
--
--
-- CHANGE HISTORY:
--     12 MAR 95   SAIC    Initial version
--     26 JUN 95   SAIC    Added test per reviewer suggestion.
--
--!

package Cxe4001_Decl_Pure is
   pragma Pure;
   Visible_User_Defined_Exception : exception;

   -- The setting of the following flag affects how much output is produced
   -- when the test runs. This output can be used to help debug problems with
   -- the test.
   Verbose : constant Boolean := False;
end Cxe4001_Decl_Pure;
